// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>	/* for EXIT_FAILURE */
#include <string.h>
#include "sim.h"

double get_mat_param(struct layer* in,int param)
{
if (in->mat[param].len==0)
{
printf("No data!\n");
exit(0);
}
//printf("%d\n",in->mat[param].len);
return in->mat[param].c[0];
}

void mat_load_poly(FILE *file,struct param* in)
{
char buffer[1000];
fgets(buffer, 1000, file);
fscanf(file,"%d",&(in->len));
//printf("segment of length= %d %s\n",in->len,buffer);
in->c = malloc(in->len*sizeof(double));
int i;
for (i=0;i<in->len;i++)
{
	fscanf(file,"%le",&(in->c[i]));
}

}

void trimstring(char * in)
{
int i;
for (i=0;i<strlen(in);i++)
{
if (in[i]==10) in[i]=0;
}
}

void load_mat_file(char *name,struct layer* in)
{
FILE *file;
if (!(file=fopen(name,"r")))
{
	printf("Error opening file %s\n",name);
	exit(0);
}
char buffer[1000];

int i;
for (i=0;i<100;i++)
{
in->mat[i].len=0;
in->mat[i].c=NULL;
}

do
{
	fgets(buffer, 1000, file);
	trimstring(buffer);
	//printf("-%s-\n",buffer);
	//getchar();
	if (strcmp(buffer,"[Eg]")==0)
	{
		//printf("found Eg\n");
		mat_load_poly(file,&(in->mat[mat_Eg]));
	}else
	if (strcmp(buffer,"[Xi]")==0)
	{
		//printf("found Xi\n");
		mat_load_poly(file,&(in->mat[mat_Xi]));
	}else
	if (strcmp(buffer,"[mue]")==0)
	{
		//printf("found mue\n");
		mat_load_poly(file,&(in->mat[mat_mue]));
	}else
	if (strcmp(buffer,"[muh]")==0)
	{
		//printf("found mun\n");
		mat_load_poly(file,&(in->mat[mat_muh]));
	}else
	if (strcmp(buffer,"[me]")==0)
	{
		//printf("found mun\n");
		mat_load_poly(file,&(in->mat[mat_me]));
	}else
	if (strcmp(buffer,"[mh]")==0)
	{
		//printf("found mun\n");
		mat_load_poly(file,&(in->mat[mat_mh]));
	}else
	if (strcmp(buffer,"[epsilonr]")==0)
	{
		//printf("found mun\n");
		mat_load_poly(file,&(in->mat[mat_epsilonr]));
	}else
	if (strcmp(buffer,"[Dex]")==0)
	{
		//printf("found mun\n");
		mat_load_poly(file,&(in->mat[mat_Dex]));
	}else
	if (strcmp(buffer,"[kf]")==0)
	{
		//printf("found mun\n");
		mat_load_poly(file,&(in->mat[mat_kf]));
	}else
	if (strcmp(buffer,"[a]")==0)
	{
		//printf("found mun\n");
		mat_load_poly(file,&(in->mat[mat_a]));
	}

}while(!feof(file));

}
