// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include "umfpack.h"
#include "sim.h"
#include <math.h>


void solver_cmp_matrix(int col,int nz,int *Ti,int *Tj, double *Tx,double *b)
{
FILE *in;
in=fopen("/home/roderick/laser/1300/matrix.dat","r");

FILE *out;
out=fopen("./diff.dat","w");

int i;
int in_col;
int in_nz;
fscanf(in,"%d",&in_nz);
fscanf(in,"%d",&in_col);

int *in_Ti = malloc(sizeof(int)*in_nz);
int *in_Tj = malloc(sizeof(int)*in_nz);
double *in_Tx = malloc(sizeof(double)*in_nz);
double *in_b = malloc(sizeof(double)*in_col);

for (i=0;i<in_nz;i++)
{
	fscanf(in,"%d %d %le",&in_Ti[i],&in_Tj[i],&in_Tx[i]);
}

int temp;
for (i=0;i<in_col;i++)
{
	fscanf(in,"%d %d %le",&temp,&temp,&in_b[i]);
}

int ii;


//	fscanf(in,"%d %d %le",&Ti[i],&Tj[i],&Tx[i]);
int found;
double diff;
for (i=0;i<nz;i++)
{
found=FALSE;

//printf("%d %d\n",Ti[i],Tj[i]);
//getchar();
	for (ii=0;ii<in_nz;ii++)
	{

		if ((in_Ti[ii]==Ti[i])&&(in_Tj[ii]==Tj[i]))
		{
			diff=in_Tx[ii]-Tx[i];
			diff=(mod(diff)/mod(in_Tx[ii]))*100.0;
			fprintf(out,"%d %d %e\n",Ti[i],Tj[i],diff);
			//found=TRUE;
			//getchar();
		}
	}
	//if (found==FALSE)
	//{
		//printf("Not found %d %d\n",Ti[i],Tj[i]);
	//}

}

for (i=0;i<in_col;i++)
{
	diff=in_b[i]-b[i];
	diff=(mod(diff)/mod(in_b[i]))*100.0;
	fprintf(out,"%d %d %e\n",0,i,diff);

}

fclose(in);
fclose(out);
free(in_Ti);
free(in_Tj);
free(in_Tx);
printf("Finished compare\n");
}

void error_report(int status, const char *file, const char *func, int line)
{
	fprintf(stderr, "in %s: file %s, line %d: ", func, file, line);

	switch (status) {
		case UMFPACK_ERROR_out_of_memory:
			fprintf(stderr, "out of memory!\n");
			break;
		case UMFPACK_WARNING_singular_matrix:
			fprintf(stderr, "matrix is singular!\n");
			break;
		default:
			fprintf(stderr, "UMFPACK error code %d\n", status);
	}
}

void solver_precon(int col,int nz,int *Ti,int *Tj, double *Tx,double *b)
{
int i;
int ii;

for (i=0;i<nz;i++)
{
	if (Ti[i]==Tj[i])
	{
		if (Tx[i]!=0.0)
		{
			b[Ti[i]]/=Tx[Ti[i]];
			for (ii=0;ii<nz;ii++)
			{
				if (Ti[ii]==Ti[i])
				{
					Tx[ii]/=Tx[Ti[i]];
					//printf("%e % ");
				}
			}
			//getchar();
		}
	}
}

}


void solver_dump_matrix(int col,int nz,int *Ti,int *Tj, double *Tx,double *b)
{
FILE *matrix;
matrix=fopen("./matrix.dat","w");
//fprintf(matrix,"%d\n",nz);
//fprintf(matrix,"%d\n",col);


int i;
for (i=0;i<nz;i++)
{
	fprintf(matrix,"%d %d %le\n",Tj[i],Ti[i],fabs(Tx[i]));
}

for (i=0;i<col;i++)
{
	fprintf(matrix,"%d %d %le\n",0,i,fabs(b[i]));
}

printf("Matrix dumped\n");

fclose(matrix);
}

int solver(int col,int nz,int *Ti,int *Tj, double *Tx,double *b)
{
int i;
void *Symbolic, *Numeric;
int status;

double *x = malloc(col*sizeof(double));
int *Ap = malloc((col+1)*sizeof(int));
int *Ai = malloc((nz)*sizeof(int));
double *Ax  = malloc((nz)*sizeof(double));

status = umfpack_di_triplet_to_col(col, col, nz, Ti, Tj, Tx, Ap, Ai, Ax, NULL);


if (status != UMFPACK_OK) {
	error_report(status, __FILE__, __func__, __LINE__);
	return EXIT_FAILURE;
}

/* symbolic analysis */
status = umfpack_di_symbolic(col, col, Ap, Ai, Ax, &Symbolic, NULL, NULL);


if (status != UMFPACK_OK) {
	error_report(status, __FILE__, __func__, __LINE__);
	return EXIT_FAILURE;
}

/* LU factorization */
umfpack_di_numeric(Ap, Ai, Ax, Symbolic, &Numeric, NULL, NULL);


if (status != UMFPACK_OK) {
	error_report(status, __FILE__, __func__, __LINE__);
	return EXIT_FAILURE;
}

umfpack_di_free_symbolic(&Symbolic);

/* solve system */
umfpack_di_solve(UMFPACK_A, Ap, Ai, Ax, x, b, Numeric, NULL, NULL);

if (status != UMFPACK_OK) {
	error_report(status, __FILE__, __func__, __LINE__);
	return EXIT_FAILURE;
}

umfpack_di_free_numeric(&Numeric);

for (i = 0; i < col; i++)
{
	b[i]=x[i];
	//printf("x[%d] = %g\n", i, x[i]);
}

free(x);
free(Ap);
free(Ai);
free(Ax);
return 0;
}
