// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

void black()
{
int i;
double E;
double Ts=3060;
double Estart=0.0e-19;//1e-7;
E=Estart;
int points=1000;
double Eend=5.0*1.6e-19;//2000e-9;
double dE=(Eend-Estart)/((double)points);
double *spec;
double b;
spec=malloc(sizeof(double)*points);
FILE *out;
out=fopen("./sun.dat","w");
for (i=0;i<points;i++)
{
	b=(2*pi*2.16e-5)/(pow(hp,3)*cl*cl);
	b*=E*E/(exp(E/(kb*Ts))-1);
	spec[i]=b;

	fprintf(out,"%e %e %e\n",E/1.6e-19,(hp*cl/E),b);
	E+=dE;
}
fclose(out);
}

void read_sun(struct light *light)
{
FILE *in;
in=fopen("./ASTMG173.csv","r");
char buf[1000];
fgets(buf,1000,in);
fgets(buf,1000,in);
fgets(buf,1000,in);

double w;
double v;
double a;
double b;
int i=0;
do
{
fscanf(in,"%lf %le %le %le",&w,&v,&a,&b);
//printf("%le %le %d\n",w,v,i);
i++;
}while(!feof(in));
rewind(in);
fgets(buf,1000,in);
fgets(buf,1000,in);
fgets(buf,1000,in);
light->sun=malloc(sizeof(double)*i);
light->x=malloc(sizeof(double)*i);
light->len=i;
i=0;
do
{
	fscanf(in,"%lf %le %le %le",&(light->x[i]),&(light->sun[i]),&a,&b);
	light->x[i]*=1e-9;
	i++;
}while(!feof(in));
double tot=0.0;
double dx=light->x[1]-light->x[0];
for (i=0;i<light->len;i++)
{
tot+=light->sun[i]*dx;
}

//printf("%lf",tot);

for (i=0;i<light->len;i++)
{
light->sun[i]/=tot;
}


fclose(in);
}

double light_get_photons(struct light *light,double power,double Eg)
{
double photons=0.0;
double E;
double dx=light->x[1]-light->x[0];
int i;
double tot_power=0.0;
for (i=0;i<light->len;i++)
{

	E=hp*cl/light->x[i];
	if (E>Eg)
	{
		photons+=light->sun[i]*power*dx/E;
		tot_power+=light->sun[i]*power*dx;
	}
}
//printf("power= %e\n",tot_power);
return photons;
}
